@@ -1,13 +1,37 @@
+"""
+The MIT License (MIT)
+
+Copyright (c) 2021-2024 null8626
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+SOFTWARE.
+"""
+
 from aiohttp import ClientSession, ClientTimeout, TCPConnector
 from urllib.parse import quote_plus
-from typing import Optional
+from typing import Optional, Tuple
 from asyncio import sleep
 from enum import auto
 
+from .errors import Error, RequestError
 from .constants import _Unit, METRIC
 from .base import CustomizableBase
 from .forecast import Forecast
-from .errors import Error
 from .enums import Locale
 
 
@@ -22,10 +46,10 @@ class Client(CustomizableBase):
   :param session: Whether to use an existing aiohttp client session for requesting or not. Defaults to ``None`` (creates a new one instead)
   :type session: Optional[:class:`aiohttp.ClientSession`]
 
-  :raises Error: If the ``unit`` argument is not ``None`` and it's also not ``METRIC`` or ``IMPERIAL``, or if the ``locale`` argument is not ``None`` and it's also not a part of the :class:`Locale` enum.
+  :raises Error: If ``unit`` is not ``METRIC`` or ``IMPERIAL``, or if ``locale`` is not ``None`` and not a part of the :class:`Locale` enum.
   """
 
-  __slots__ = ('__session',)
+  __slots__: Tuple[str, ...] = ('__own_session', '__session')
 
   def __init__(
     self,
@@ -36,13 +60,15 @@ class Client(CustomizableBase):
   ):
     super().__init__(unit, locale)
 
+    self.__own_session = session is None
     self.__session = session or ClientSession(
+      raise_for_status=True,
       timeout=ClientTimeout(total=5000.0),
       connector=TCPConnector(verify_ssl=False),
     )
 
   def __repr__(self) -> str:
-    return f'<{self.__class__.__name__} {self.__session!r}>'
+    return f'<{__class__.__name__} {self.__session!r}>'
 
   async def get(
     self,
@@ -56,9 +82,9 @@ class Client(CustomizableBase):
 
     :param location: The requested location name for said weather forecast.
     :type location: str
-    :param unit: Overrides the metric or imperial/customary system (``IMPERIAL``) used by the :class:`Client` object. Defaults to ``None`` (uses the one from the :class:`Client`).
+    :param unit: Overrides the unit used by this object. Defaults to the one used by this object.
     :type unit: Optional[:py:class:`enum.auto`]
-    :param locale: Overrides the locale/language used by the :class:`Client` object. Defaults to ``None`` (uses the one from the :class:`Client`).
+    :param locale: Overrides the locale used by this object. Defaults to the one used by this object.
     :type locale: Optional[Locale]
 
     :exception Error: If the aiohttp client session used by the :class:`Client` object is already closed, if the ``unit`` argument is not ``None`` and it's also not ``METRIC`` or ``IMPERIAL``, if the ``locale`` argument is not ``None`` and it's also not a part of the :class:`Locale` enum, or if the :class:`Client` cannot send a web request to the web server.
@@ -82,25 +108,25 @@ class Client(CustomizableBase):
     delay = 0.5
 
     while True:
-      async with self.__session.get(
-        f'https://{subdomain}wttr.in/{quote_plus(location)}?format=j1'
-      ) as resp:
-        try:
+      try:
+        async with self.__session.get(
+          f'https://{subdomain}wttr.in/{quote_plus(location)}?format=j1'
+        ) as resp:
           return Forecast(await resp.json(), unit, locale)
-        except Exception as e:
-          if delay == 2:
-            raise e  # okay, that's too much requests - just raise the error
+      except Exception as err:
+        if delay == 4:
+          raise RequestError(err)
 
-          await sleep(delay)
-          delay *= 2
+        await sleep(delay)
+        delay *= 2
 
-  async def close(self):
-    """Closes the :class:`Client` object. Nothing will happen if it's already closed."""
+  async def close(self) -> None:
+    """Closes the :class:`Client` object. Nothing will happen if the client uses a pre-existing :class:`aiohttp.ClientSession` or if the session is already closed."""
 
-    if not self.__session.closed:
+    if self.__own_session and not self.__session.closed:
       await self.__session.close()
 
-  async def __aenter__(self):
+  async def __aenter__(self) -> 'Client':
     return self
 
   async def __aexit__(self, *_, **__):
